@@ -1,6 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright 2012 OpenStack LLC
+# Copyright 2013 IBM Corp.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -33,6 +34,9 @@ CONF = config.CONF
 
 
 class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
+    def _get_domain_fixture(self):
+        """Domains in LDAP are read-only, so just return the static one."""
+        return self.identity_api.get_domain(CONF.identity.default_domain_id)
 
     def clear_database(self):
         db = fakeldap.FakeShelve().get_instance()
@@ -373,8 +377,11 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
     # In the tests below, the update is demonstrated by updating description.
     # Refer to bug 1136403 for more detail.
     def test_group_crud(self):
-        group = {'id': uuid.uuid4().hex, 'domain_id': uuid.uuid4().hex,
-                 'name': uuid.uuid4().hex, 'description': uuid.uuid4().hex}
+        group = {
+            'id': uuid.uuid4().hex,
+            'domain_id': CONF.identity.default_domain_id,
+            'name': uuid.uuid4().hex,
+            'description': uuid.uuid4().hex}
         self.identity_api.create_group(group['id'], group)
         group_ref = self.identity_api.get_group(group['id'])
         self.assertDictEqual(group_ref, group)
@@ -391,15 +398,25 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
     def test_domain_crud(self):
         domain = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex,
                   'enabled': True, 'description': uuid.uuid4().hex}
-        self.identity_api.create_domain(domain['id'], domain)
-        domain_ref = self.identity_api.get_domain(domain['id'])
-        self.assertDictEqual(domain_ref, domain)
-        domain['description'] = uuid.uuid4().hex
-        self.identity_api.update_domain(domain['id'], domain)
-        domain_ref = self.identity_api.get_domain(domain['id'])
-        self.assertDictEqual(domain_ref, domain)
-
-        self.identity_api.delete_domain(domain['id'])
+        with self.assertRaises(exception.Forbidden):
+            self.identity_api.create_domain(domain['id'], domain)
+        with self.assertRaises(exception.Conflict):
+            self.identity_api.create_domain(
+                CONF.identity.default_domain_id, domain)
+        with self.assertRaises(exception.DomainNotFound):
+            domain_ref = self.identity_api.get_domain(domain['id'])
+        with self.assertRaises(exception.DomainNotFound):
+            domain['description'] = uuid.uuid4().hex
+            self.identity_api.update_domain(domain['id'], domain)
+        with self.assertRaises(exception.Forbidden):
+            self.identity_api.update_domain(
+                CONF.identity.default_domain_id, domain)
+        with self.assertRaises(exception.DomainNotFound):
+            self.identity_api.get_domain(domain['id'])
+        with self.assertRaises(exception.DomainNotFound):
+            self.identity_api.delete_domain(domain['id'])
+        with self.assertRaises(exception.Forbidden):
+            self.identity_api.delete_domain(CONF.identity.default_domain_id)
         self.assertRaises(exception.DomainNotFound,
                           self.identity_api.get_domain,
                           domain['id'])
@@ -420,10 +437,10 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
         raise nose.exc.SkipTest('Blocked by bug 1101287')
 
     def test_get_and_remove_role_grant_by_group_and_domain(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_get_and_remove_role_grant_by_user_and_domain(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_get_and_remove_correct_role_grant_from_a_mix(self):
         raise nose.exc.SkipTest('Blocked by bug 1101287')
@@ -433,7 +450,7 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
         #              updating of a project name so this method override
         #              provides a different update test
         project = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex,
-                   'domain_id': uuid.uuid4().hex,
+                   'domain_id': CONF.identity.default_domain_id,
                    'description': uuid.uuid4().hex
                    }
         self.identity_api.create_project(project['id'], project)
@@ -456,34 +473,34 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
                           project['id'])
 
     def test_get_and_remove_role_grant_by_group_and_cross_domain(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_get_and_remove_role_grant_by_user_and_cross_domain(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_role_grant_by_group_and_cross_domain_project(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_role_grant_by_user_and_cross_domain_project(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_multi_role_grant_by_user_group_on_project_domain(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_delete_role_with_user_and_group_grants(self):
         raise nose.exc.SkipTest('Blocked by bug 1101287')
 
     def test_delete_user_with_group_project_domain_links(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_delete_group_with_user_project_domain_links(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_list_user_projects(self):
         raise nose.exc.SkipTest('Blocked by bug 1101287')
 
     def test_get_project_users(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
 
     def test_create_duplicate_user_name_in_different_domains(self):
         raise nose.exc.SkipTest('Blocked by bug 1101276')
@@ -492,7 +509,8 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
         raise nose.exc.SkipTest('Blocked by bug 1101276')
 
     def test_create_duplicate_group_name_in_different_domains(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101276')
+        raise nose.exc.SkipTest(
+            'N/A: LDAP does not support multiple domains')
 
     def test_move_user_between_domains(self):
         raise nose.exc.SkipTest('Blocked by bug 1101276')
@@ -501,7 +519,8 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
         raise nose.exc.SkipTest('Blocked by bug 1101276')
 
     def test_move_group_between_domains(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101276')
+        raise nose.exc.SkipTest(
+            'N/A: LDAP does not support multiple domains')
 
     def test_move_group_between_domains_with_clashing_names_fails(self):
         raise nose.exc.SkipTest('Blocked by bug 1101276')
@@ -513,7 +532,50 @@ class LDAPIdentity(test.TestCase, test_backend.IdentityTests):
         raise nose.exc.SkipTest('Blocked by bug 1101276')
 
     def test_get_roles_for_user_and_domain(self):
-        raise nose.exc.SkipTest('Blocked by bug 1101287')
+        raise nose.exc.SkipTest('N/A: LDAP does not support multiple domains')
+
+    def test_list_group_members_missing_entry(self):
+        """List group members with deleted user.
+
+        If a group has a deleted entry for a member, the non-deleted members
+        are returned.
+
+        """
+
+        # Create a group
+        group_id = None
+        group = dict(name=uuid.uuid4().hex)
+        group_id = self.identity_api.create_group(group_id, group)['id']
+
+        # Create a couple of users and add them to the group.
+        user_id = None
+        user = dict(name=uuid.uuid4().hex, id=uuid.uuid4().hex)
+        user_1_id = self.identity_api.create_user(user_id, user)['id']
+
+        self.identity_api.add_user_to_group(user_1_id, group_id)
+
+        user_id = None
+        user = dict(name=uuid.uuid4().hex, id=uuid.uuid4().hex)
+        user_2_id = self.identity_api.create_user(user_id, user)['id']
+
+        self.identity_api.add_user_to_group(user_2_id, group_id)
+
+        # Delete user 2.
+        self.identity_api.user.delete(user_2_id)
+
+        # List group users and verify only user 1.
+        res = self.identity_api.list_users_in_group(group_id)
+
+        self.assertEqual(len(res), 1, "Expected 1 entry (user_1)")
+        self.assertEqual(res[0]['id'], user_1_id, "Expected user 1 id")
+
+    def test_list_domains(self):
+        domains = self.identity_api.list_domains()
+        self.assertEquals(
+            domains,
+            [{'id': CONF.identity.default_domain_id,
+              'name': 'Default',
+              'enabled': True}])
 
 
 class LDAPIdentityEnabledEmulation(LDAPIdentity):
@@ -560,10 +622,11 @@ class LDAPIdentityEnabledEmulation(LDAPIdentity):
         # NOTE(topol): LDAPIdentityEnabledEmulation will create an
         #              enabled key in the project dictionary so this
         #              method override handles this side-effect
-        project = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex,
-                   'domain_id': uuid.uuid4().hex,
-                   'description': uuid.uuid4().hex
-                   }
+        project = {
+            'id': uuid.uuid4().hex,
+            'name': uuid.uuid4().hex,
+            'domain_id': CONF.identity.default_domain_id,
+            'description': uuid.uuid4().hex}
 
         self.identity_api.create_project(project['id'], project)
         project_ref = self.identity_api.get_project(project['id'])
@@ -585,8 +648,11 @@ class LDAPIdentityEnabledEmulation(LDAPIdentity):
                           project['id'])
 
     def test_user_crud(self):
-        user = {'domain_id': uuid.uuid4().hex, 'id': uuid.uuid4().hex,
-                'name': uuid.uuid4().hex, 'password': 'passw0rd'}
+        user = {
+            'id': uuid.uuid4().hex,
+            'domain_id': CONF.identity.default_domain_id,
+            'name': uuid.uuid4().hex,
+            'password': uuid.uuid4().hex}
         self.identity_man.create_user({}, user['id'], user)
         user['enabled'] = True
         user_ref = self.identity_api.get_user(user['id'])
